"""ATMultiEditTool.py

Mailback password reset product for CMF.
Author: J Cameron Cooper, Sept 2003
"""

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View, ManagePortal, ModifyPortalContent
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from interfaces.at_multi_edit import portal_multiedit as IATMultiEditTool

class ATMultiEditTool (UniqueObject, ActionProviderBase, SimpleItem):
    """ Tool that allows processing of mare that one archetype object """

    __implements__ = (IATMultiEditTool)

    id = 'portal_multiedit'
    meta_type = 'Multi Edit Tool'

    _actions = [ActionInformation(
        id='multi_edit'
        , title='Multi Edit'
        , action='string: ${portal_url}/portal_multiedit/multi_edit'
        , condition='python:1'
        , permissions=(ModifyPortalContent,)
        , category='folder_buttons'
        , visible=0
        )]

    security = ClassSecurityInfo()

    #security.declareProtected(ModifyPortalContent,'rebuildREQUEST')
    security.declarePublic('rebuildREQUEST')
    def rebuildREQUEST(self,REQUEST=None,obj=None,form_prefixes=[]):
        """ returns a reconfigured request """

        prefix = obj.UID()

        #cleanup
        fields = obj.Schema().filterFields(isMetadata=0)
        fieldNames = [f.getName() for f in fields]
        for f in fieldNames:
            try:
                del REQUEST.form[f]
            except:
                pass

        other_form_prefixes = [p for p in form_prefixes]
        other_form_prefixes.remove(prefix)
        for key in REQUEST.form.keys():
            if key.startswith(prefix):
                k = key[len(prefix):]
                v = REQUEST.form.get(key)
                REQUEST.form[k]=v
            elif [p for p in other_form_prefixes if key.startswith(p)] == []:
                k = key
                v = REQUEST.form.get(k)
                REQUEST.form[k]=v
            else:
                pass

        REQUEST.form['id'] = obj.getId()
        return REQUEST

    manage_options=(( { 'label' : 'Overview'
                        , 'action' : 'manage_overview'
                        },
                      ) + SimpleItem.manage_options
                        + ActionProviderBase.manage_options
                    )

    ##   ZMI methods
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('dtml/explainATMultiEditTool', globals() )

    def __init__(self):
        pass

InitializeClass(ATMultiEditTool)
