# -*- coding: utf-8 -*-
#
# File: Cart.py
#
# Copyright (c) 2006 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMallRegistry.ICart import ICart
from Products.Archetypes.AllowedTypesByIface import AllowedTypesByIfaceMixin
from Products.PloneMallRegistry.ICartItem import ICartItem
from Products.PloneMallCart.config import *

##code-section module-header #fill in your manual code here
from Products.PloneMallRegistry.util import filterContents
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from AccessControl.unauthorized import Unauthorized
from Products.PloneMallRegistry.ICartItem import ICartItem
from Products.PloneMallRegistry.Currency import Currency
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.SkinnedFolder  import SkinnedFolder
from Products.Archetypes.debug import deprecated

##/code-section module-header

schema = Schema((

    ComputedField(
        name='title',
        widget=ComputedField._properties['widget'](
            label='Title',
            label_msgid='PloneMallCart_label_title',
            i18n_domain='PloneMallCart',
        ),
        expression="'Cart #%s' % context.id"
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

Cart_schema = BaseFolderSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
from OFS.ObjectManager import ObjectManager
##/code-section after-schema

class Cart(AllowedTypesByIfaceMixin, BaseFolder):
    """Cart
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(AllowedTypesByIfaceMixin,'__implements__',()),) + (getattr(BaseFolder,'__implements__',()),) + (ICart,)

    # This name appears in the 'add' box
    archetype_name = 'Cart'

    meta_type = 'Cart'
    portal_type = 'Cart'
    allowed_content_types = []
    allowed_interfaces = [ICartItem] 
    filter_content_types = 1
    global_allow = 0
    #content_icon = 'Cart.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "Cart"
    typeDescMsgId = 'description_edit_cart'


    actions =  (


       {'action': "string:${object_url}/cart_overview",
        'category': "object",
        'id': 'view',
        'name': 'view',
        'permissions': ("View",),
        'condition': 'python:1'
       },


    )

    _at_rename_after_creation = False

    schema = Cart_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    # Methods from Interface ICart

    security.declareProtected("View", 'getContents')
    def getContents(self, filter = {}):
        '''
        Return a list of objects that are inside the cart. The result
        could be filtered using a dictionary, where the keys are object
        properties and the values are acceptable values for these keys.
        '''

        return filterContents(self.contentValues(), filter)

    security.declareProtected("View", 'getTotal')
    def getTotal(self):
        '''Returns the carts total.

        A currency object with the value summation of all items is returned
        '''
        ret = Currency(context=self)
        for obj in self.getContents():
            ret += obj.getTotal()
        return ret

    security.declareProtected("Buy item", 'updateItem')
    def updateItem(self, cart_item, quantity):
        '''
        Update item quantity in the cart.
        '''

        self._checkSessionId()

        if not FactoryRegistry.objectSupportsInterface(cart_item, ICartItem):
            raise TypeError("cart item %s is not registered as a ICartItem implementation" % \
                cart_item.__class__.__name__)

        if quantity < 0:
            raise ValueError("'%d' is not a valid quantity." % quantity)

        if quantity == 0:
            self.delItem([cart_item.getId()])
        else:
            cart_item.setQuantity(quantity)

    security.declareProtected("Buy item", 'addItem')
    def addItem(self, item, quantity=1):
        '''
        Add item to cart.
        '''

        self._checkSessionId()

        ci = None
        l_contents = self.getContents()
        for obj in l_contents:
            if obj.getMallItem() == item:
                ci = obj
                break
        if ci is None:
            id = str(len(l_contents) + 1)

            mall_factory=getToolByName(self,'portal_mall_registry')
            mall_factory.constructContent(ICartItem,self,id)
            ci = getattr(self, id)
            ci.setMallItem(item)
            ci.setQuantity(quantity)
            ci.setPrice(item.getPrice().getValue())
            ci.setCurrency(item.getPrice().getCurrencySymbol())
        else:
            # For now, when we add the same item to the cart more than
            # one time, we will add 1 to the item quantity.  In a
            # future implementation, where we would be worried about
            # different shipping for each cart item, this method
            # should be modified to not add to the quantity, but
            # allways add a new cart item.
            ci.setQuantity(ci.getQuantity() + quantity)

    security.declareProtected("Buy item", 'delItem')
    def delItem(self, cart_item):
        '''
        Delete item from the cart.
        '''

        self._checkSessionId()

        # XXX: hack to circumvent Unauthorized errors on cart delete
        # self.manage_delObjects(cart_item)
        ObjectManager.manage_delObjects(self, cart_item)

        # keep the cart items ids as a 'well formed' sequence
        # This works for an anonymous user as long as the security settings are
        # set to allow 'Add portal content' for anonymous. (As they are in the
        # cartmanager workflow )
        # We _do_ need to reorganize the sequence as long as addItem depends on
        # it.
        old_ids = [obj.getId() for obj in self.getContents()]
        new_ids = [str(n + 1) for n in range(len(old_ids))]
        if old_ids != new_ids:
            self.manage_renameObjects(old_ids, new_ids)

    security.declarePublic('isFilled')
    def isFilled(self):
        """
        if theres at least one item in the cart True is returned
        """
        return len(self.contentIds()) > 0

    # Manually created methods

    def __before_publishing_traverse__(self, object, REQUEST):
        """Add hook to post_traverse, to check for matching session id when
        traversing to the cart object.
        """
        #add post traversal hook to check session id:
        REQUEST.post_traverse(*(self._checkSessionIdHook, ))

    security.declarePublic('getOrder')
    def getOrder(self):
        """
        deprecated method to return the order
        """
        deprecated('This method was a design mistake. The cart has no contract '
                   'with the order. its vice versa.')
        # XXX: This is currently used to get an existing order for a session, via
        # the cart. How shall that work in the future?
        orders=self.getBRefs('order_cart_object')
        assert len(orders)<=1, '%s order connected to the cart, maximum 1 allowed' % len(orders)
        if orders:
            return orders[0]
        else:
            return None

    def _checkSessionIdHook(self):
        membership = getToolByName(self, 'portal_membership')

        if not membership.checkPermission('Manage portal', self):
            self._checkSessionId()

    def _checkSessionId(self):

        # For security reasons, enforce that the REQUEST comes from
        # the same user that created the Cart. It is important to
        # avoid that other Anonymous user or a Member add items into
        # Cart of others.

        sid = self.REQUEST.SESSION.getId()
        cid = self.getId()
        if sid != cid:
            raise Unauthorized


registerType(Cart, PROJECTNAME)
# end of class Cart

##code-section module-footer #fill in your manual code here
if register_cart:
    FactoryRegistry.registerMallFactory('Cart', Cart, (ICart,))
##/code-section module-footer



