# -*- coding: utf-8 -*-
#
# File: CartItem.py
#
# Copyright (c) 2006 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMallRegistry.ICartItem import ICartItem
from Products.PloneMallCart.config import *

# additional imports from tagged value 'import'
from Products.PloneMallRegistry.Widget import DecimalL10nWidget

##code-section module-header #fill in your manual code here
from Products.PloneMallRegistry.Currency import Currency, VOC_CURRENCY
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.IMallItem import IMallItem
from Products.CMFCore.utils import getToolByName
##/code-section module-header

schema = Schema((

    ReferenceField(
        name='item',
        widget=ReferenceField._properties['widget'](
            label='Item',
            label_msgid='PloneMallCart_label_item',
            i18n_domain='PloneMallCart',
        ),
        allowed_types=('SimpleItem','ItemVariant'),
        relationship="item",
        required=1
    ),

    StringField(
        name='currency',
        widget=SelectionWidget
        (
            label='Currency',
            label_msgid='PloneMallCart_label_currency',
            i18n_domain='PloneMallCart',
        ),
        required=1,
        vocabulary=VOC_CURRENCY,
        default_method="getDefaultCurrency"
    ),

    FixedPointField(
        name='item_price',
        widget=DecimalL10nWidget
        (
            label="Price",
            label_msgid='PloneMallCart_label_item_price',
            i18n_domain='PloneMallCart',
        ),
        required=1,
        validators=('isDecimalL10n',)
    ),

    FixedPointField(
        name='item_quantity',
        widget=DecimalWidget(
            label="Quantity",
            label_msgid='PloneMallCart_label_item_quantity',
            i18n_domain='PloneMallCart',
        ),
        required=1
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

CartItem_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class CartItem(BaseFolder):
    """Cart Item
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseFolder,'__implements__',()),) + (ICartItem,)

    # This name appears in the 'add' box
    archetype_name = 'CartItem'

    meta_type = 'CartItem'
    portal_type = 'CartItem'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 0
    #content_icon = 'CartItem.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "CartItem"
    typeDescMsgId = 'description_edit_cartitem'

    _at_rename_after_creation = False

    schema = CartItem_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    # Methods from Interface ICartItem

    security.declareProtected("View", 'getQuantity')
    def getQuantity(self):
        '''
        Return the product quantity.
        '''

        return float(self.getItem_quantity())

    security.declareProtected("View", 'getTotal')
    def getTotal(self):
        '''
        Return the cart item total.
        '''

        return self.getPrice() * self.getQuantity()

    security.declareProtected("Modify portal content", 'setPrice')
    def setPrice(self, price):
        '''
        Set the product price.
        '''

        ## test if price is something like 'US$ 10.99'
        l = price.split(' ')
        if len(l) == 2:
            self.setCurrency(l[0])
            self.setItem_price(l[1])
        else:
            self.setItem_price(l[0])

    security.declareProtected("Modify portal content", 'setMallItem')
    def setMallItem(self, item):
        '''
        Set the product.
        '''

        if not FactoryRegistry.objectSupportsInterface(item, IMallItem):
            raise TypeError("item %s is not registered as a IMallItem implementation" % \
                item.__class__.__name__)

        self.deleteReferences('item')
        self.addReference(item, 'item')
        self.setTitle(item.getIdentifier())

    security.declareProtected("Modify portal content", 'setQuantity')
    def setQuantity(self,quantity):
        '''
        Set the product quantity.
        '''

        self.setItem_quantity(str(quantity))

    security.declareProtected("View", 'getPrice')
    def getPrice(self):
        '''
        Return the product price.
        '''

        return Currency(self.getCurrency(), self.getItem_price(), context=self)

    security.declareProtected("View", 'getMallItem')
    def getMallItem(self):
        '''
        Return the product.
        '''

        try:
            return self.getRefs('item')[0]
        except IndexError:
            return None

    # Manually created methods

    def getDefaultCurrency(self):
        return self.portal_mall_registry.getDefaultCurrency()



registerType(CartItem, PROJECTNAME)
# end of class CartItem

##code-section module-footer #fill in your manual code here
if register_cart:
   FactoryRegistry.registerMallFactory('CartItem', CartItem, (ICartItem,))
##/code-section module-footer



