# -*- coding: utf-8 -*-
#
# File: CartManager.py
#
# Copyright (c) 2006 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMallRegistry.ICartManager import ICartManager
from Products.Archetypes.AllowedTypesByIface import AllowedTypesByIfaceMixin
from Products.PloneMallRegistry.ICart import ICart
from Products.PloneMallCart.config import *


from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
from Products.CMFCore.utils import getToolByName

from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.Archetypes.debug import deprecated

##/code-section module-header

schema = Schema((

    StringField(
        name='select_mode',
        default="checkbox",
        widget=SelectionWidget
        (
            description="Show checkboxes or editboxes with each item",
            label="Select Mode",
            label_msgid='PloneMallCart_label_select_mode',
            description_msgid='PloneMallCart_help_select_mode',
            i18n_domain='PloneMallCart',
        ),
        vocabulary=DisplayList((('checkbox','Checkbox'),('numeric','Numeric')))
    ),

    StringField(
        name='add_to_cart_mode',
        default="stay",
        widget=SelectionWidget
        (
            description="Stay on the current context or show the cart when adding items.",
            label="Add to cart mode",
            label_msgid='PloneMallCart_label_add_to_cart_mode',
            description_msgid='PloneMallCart_help_add_to_cart_mode',
            i18n_domain='PloneMallCart',
        ),
        read_permission="getOrCreateCart",
        vocabulary=DisplayList((('stay','Stay on context'),('show_cart','Show cart')))
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

CartManager_schema = BaseBTreeFolderSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class CartManager(UniqueObject, AllowedTypesByIfaceMixin, BaseBTreeFolder):
    """Cart Manager
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(AllowedTypesByIfaceMixin,'__implements__',()),) + (getattr(BaseBTreeFolder,'__implements__',()),) + (ICartManager,)

    # This name appears in the 'add' box
    archetype_name = 'CartManager'

    meta_type = 'CartManager'
    portal_type = 'CartManager'
    allowed_content_types = []
    allowed_interfaces = [ICart] 
    filter_content_types = 1
    global_allow = 0
    #content_icon = 'CartManager.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "CartManager"
    typeDescMsgId = 'description_edit_cartmanager'
    #toolicon = 'CartManager.gif'

    _at_rename_after_creation = False

    schema = CartManager_schema

    ##code-section class-header #fill in your manual code here
    security.declareProtected("getOrCreateCart", 'getSelect_mode')
    security.declareProtected("getOrCreateCart", 'getAdd_to_cart_mode')
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseBTreeFolder.__init__(self,'portal_cartmanager')
        self.setTitle('CartManager')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer


    # tool should not appear in portal_catalog
    def at_post_edit_script(self):
        self.unindexObject()
        
        ##code-section post-edit-method-footer #fill in your manual code here
        ##/code-section post-edit-method-footer


    # Methods

    security.declareProtected("getOrCreateCart", 'getOrCreateCart')
    def getOrCreateCart(self, create=True):
        '''
        Return an instance of the registered ICart interface, creating
        the cart at the first time and only returning it after it
        initial creation.
        '''
        session = self.session_data_manager.getSessionData(create=create)
        if session is None:
            return None
        sid = session.getId()
        cart = self._getOb(sid, None)
        if cart is None and create:
            mall_factory=getToolByName(self,'portal_mall_registry')
            mall_factory.constructContent(ICart,self,sid)

            cart = getattr(self, sid)
            cart.notifyWorkflowCreated()
        return cart

    # Manually created methods

    def hasFilledCart(self):
        '''Returns 'True' if a cart exists and cart is not empty.
        '''
        deprecated('using hasFilledCart has been deprecated due performance '
                   'issues, use method "isFilled" on the cart object itself.')
        cart = self.getOrCreateCart(create=False)
        if cart is not None:
            if cart.getContents() :
                return True
        return False

    def view(self,*a,**kw):
        '''circumvent plone bug that view doesnt redirect to the correct view method'''
        return self.base_view(*a,**kw)

    def deleteCart(self,cart):
        self.manage_delObjects(cart.id)



registerType(CartManager, PROJECTNAME)
# end of class CartManager

##code-section module-footer #fill in your manual code here
if register_cart:
   FactoryRegistry.registerMallFactory('CartManager', CartManager, (ICartManager,))
##/code-section module-footer



