# -*- coding: utf-8 -*-
#
# File: PloneMallCart.py
#
# Copyright (c) 2006 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'


from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from Products.PloneMallCart.config import *

##code-section create-workflow-module-header #fill in your manual code here
##/code-section create-workflow-module-header


productname = 'PloneMallCart'

def setupcart_workflow(self, workflow):
    """Define the cart_workflow workflow.
    """

    workflow.setProperties(title='cart_workflow')

    ##code-section create-workflow-setup-method-header #fill in your manual code here
    ##/code-section create-workflow-setup-method-header


    for s in ['closed', 'shopping']:
        workflow.states.addState(s)

    for t in ['finish_shopping']:
        workflow.transitions.addTransition(t)

    for v in ['review_history', 'comments', 'time', 'actor', 'action']:
        workflow.variables.addVariable(v)

    workflow.addManagedPermission('Add CartItem Content')
    workflow.addManagedPermission('Modify portal content')
    workflow.addManagedPermission('Access contents information')
    workflow.addManagedPermission('View')
    workflow.addManagedPermission('getOrCreateCart')

    for l in []:
        if not l in workflow.worklists.objectValues():
            workflow.worklists.addWorklist(l)

    ## Initial State

    workflow.states.setInitialState('shopping')

    ## States initialization

    stateDef = workflow.states['closed']
    stateDef.setProperties(title="""closed""",
                           description="""""",
                           transitions=[])
    stateDef.setPermission('Add CartItem Content',
                           0,
                           [])
    stateDef.setPermission('Modify portal content',
                           0,
                           [])
    stateDef.setPermission('Access contents information',
                           0,
                           ['Anonymous', 'Authenticated'])
    stateDef.setPermission('View',
                           0,
                           ['Anonymous', 'Authenticated'])

    stateDef = workflow.states['shopping']
    stateDef.setProperties(title="""shopping""",
                           description="""""",
                           transitions=['finish_shopping'])
    stateDef.setPermission('Add CartItem Content',
                           0,
                           ['Manager', 'Owner'])
    stateDef.setPermission('Modify portal content',
                           0,
                           ['Manager', 'Owner', 'Anonymous'])
    stateDef.setPermission('Access contents information',
                           0,
                           ['Anonymous', 'Authenticated'])
    stateDef.setPermission('View',
                           0,
                           ['Anonymous', 'Authenticated'])
    stateDef.setPermission('getOrCreateCart',
                           0,
                           ['Anonymous', 'Manager', 'Owner'])

    ## Transitions initialization

    ## Creation of workflow scripts
    for wf_scriptname in ['preclose', 'postclose']:
        if not wf_scriptname in workflow.scripts.objectIds():
            workflow.scripts._setObject(wf_scriptname,
                ExternalMethod(wf_scriptname, wf_scriptname,
                productname + '.cart_workflow_scripts',
                wf_scriptname))

    transitionDef = workflow.transitions['finish_shopping']
    transitionDef.setProperties(title="""finish_shopping""",
                                new_state_id="""closed""",
                                trigger_type=1,
                                script_name="""preclose""",
                                after_script_name="""postclose""",
                                actbox_name="""finish_shopping""",
                                actbox_url="""""",
                                actbox_category="""workflow""",
                                props={},
                                )

    ## State Variable
    workflow.variables.setStateVar('review_state')

    ## Variables initialization
    variableDef = workflow.variables['review_history']
    variableDef.setProperties(description="""Provides access to workflow history""",
                              default_value="""""",
                              default_expr="""state_change/getHistory""",
                              for_catalog=0,
                              for_status=0,
                              update_always=0,
                              props={'guard_permissions': 'Request review; Review portal content'})

    variableDef = workflow.variables['comments']
    variableDef.setProperties(description="""Comments about the last transition""",
                              default_value="""""",
                              default_expr="""python:state_change.kwargs.get('comment', '')""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['time']
    variableDef.setProperties(description="""Time of the last transition""",
                              default_value="""""",
                              default_expr="""state_change/getDateTime""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['actor']
    variableDef.setProperties(description="""The ID of the user who performed the last transition""",
                              default_value="""""",
                              default_expr="""user/getId""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    variableDef = workflow.variables['action']
    variableDef.setProperties(description="""The last transition""",
                              default_value="""""",
                              default_expr="""transition/getId|nothing""",
                              for_catalog=0,
                              for_status=1,
                              update_always=1,
                              props=None)

    ## Worklists Initialization


    # WARNING: below protected section is deprecated.
    # Add a tagged value 'worklist' with the worklist name to your state(s) instead.

    ##code-section create-workflow-setup-method-footer #fill in your manual code here
    ##/code-section create-workflow-setup-method-footer



def createcart_workflow(self, id):
    """Create the workflow for PloneMallCart.
    """

    ob = DCWorkflowDefinition(id)
    setupcart_workflow(self, ob)
    return ob

addWorkflowFactory(createcart_workflow,
                   id='cart_workflow',
                   title='cart_workflow')

##code-section create-workflow-module-footer #fill in your manual code here
##/code-section create-workflow-module-footer

