# -*- coding: utf-8 -*-
#
# File: AVS.py
#
# Copyright (c) 2006 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMallRegistry.IAVS import IAVS
from Products.PloneMallCustomer.config import *


from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.PloneMallRegistry.Widget import LargeMultiSelectionWidget

from ISO3166Parser import ISO3166Parser
from Globals import package_home
from Products.PloneMallCustomer import product_globals
from os.path import join

iso3166_parser = ISO3166Parser()
iso3166_path = join(package_home(product_globals), 'iso3166')
iso3166_parser.parseAll(iso3166_path)

country_codes = [c[0] for c in iso3166_parser._countries]

##/code-section module-header

schema = Schema((

    LinesField(
        name='supported_countries',
        default= country_codes,
        widget=LargeMultiSelectionWidget
        (
            label="Supported Countries",
            label_msgid='PloneMallCustomer_label_supported_countries',
            i18n_domain='PloneMallCustomer',
        ),
        vocabulary='getAllCountries'
    ),

    LinesField(
        name='preferred_countries',
        default= country_codes,
        widget=LargeMultiSelectionWidget
        (
            label="Preferred Countries",
            label_msgid='PloneMallCustomer_label_preferred_countries',
            i18n_domain='PloneMallCustomer',
        ),
        vocabulary='getAllCountries'
    ),

    StringField(
        name='DefaultCountry',
        default="US",
        widget=SelectionWidget
        (
            label="Default Country",
            label_msgid='PloneMallCustomer_label_DefaultCountry',
            i18n_domain='PloneMallCustomer',
        ),
        vocabulary='getAllCountries'
    ),

    StringField(
        name='DefaultState',
        widget=StringWidget(
            label="Default State",
            label_msgid='PloneMallCustomer_label_DefaultState',
            i18n_domain='PloneMallCustomer',
        )
    ),

    BooleanField(
        name='askLocationName',
        default="1",
        widget=BooleanField._properties['widget'](
            label="Ask Location?",
            label_msgid='PloneMallCustomer_label_askLocationName',
            i18n_domain='PloneMallCustomer',
        )
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

AVS_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class AVS(UniqueObject, BaseContent):
    """Address Verification System
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(BaseContent,'__implements__',()),) + (IAVS,)

    # This name appears in the 'add' box
    archetype_name = 'AVS'

    meta_type = 'AVS'
    portal_type = 'AVS'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 0
    #content_icon = 'AVS.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "AVS"
    typeDescMsgId = 'description_edit_avs'
    #toolicon = 'AVS.gif'

    _at_rename_after_creation = False

    schema = AVS_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_avs')
        self.setTitle('AVS')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer


    # tool should not appear in portal_catalog
    def at_post_edit_script(self):
        self.unindexObject()
        
        ##code-section post-edit-method-footer #fill in your manual code here
        ##/code-section post-edit-method-footer


    # Methods

    # Methods from Interface IAVS

    security.declareProtected("View", 'getStatesForCountry')
    def getStatesForCountry(self,country):
        '''
        Return the States of a Country as a list of tuples, where the
        first tuple position if the State code (following the ISO3166
        specification) and the second position is the State name.
        '''

        l = iso3166_parser._states[country]
        return l or [('##', 'This Country does not have States.')]

    security.declareProtected("View", 'getSupportedCountries')
    def getSupportedCountries(self, ac = None):
        '''
        Lists the countries that are supported as a list of tuples
        (iso,countryname)
        '''

        if ac is None:
            ac = dict(self.getAllCountries())
        ret = []
        for pc in self.getSupported_countries():
            ret.append((pc, ac[pc]))
        return ret

    security.declarePublic('operation_2')
    def operation_2(self):
        """

        """
        pass

    security.declareProtected("View", 'getPreferredCountries')
    def getPreferredCountries(self, ac = None):
        '''
        Lists the countries that are preferred and appear first in
        the country list as a list of tuples (iso,countryname)
        '''

        if ac is None:
            ac = dict(self.getAllCountries())
        ret = []
        for pc in self.getPreferred_countries():
            ret.append((pc, ac[pc]))
        return ret

    security.declareProtected("View", 'getCountries')
    def getCountries(self):
        '''
        Return a list of tuples, where the first tuple position is
        the coutry code (following the ISO3166 specification) and the
        second position is the Country name.
        '''

        ac = dict(self.getAllCountries())
        pref = self.getPreferredCountries(ac)
        d_pref = dict(pref)
        supp = self.getSupported_countries()
        return pref + [(s, ac[s]) for s in supp if not d_pref.has_key(s)]

    # Manually created methods

    def getAllCountries(self):
        '''
        Return a list of tuples, where the first tuple position is the
        coutry code (following the ISO3166 specification) and the second
        position is the Coutry name.
        '''

        return iso3166_parser._countries



registerType(AVS, PROJECTNAME)
# end of class AVS

##code-section module-footer #fill in your manual code here
if register_avs:
   FactoryRegistry.registerMallFactory('AVS', AVS,)
##/code-section module-footer



