# -*- coding: utf-8 -*-
#
# File: BaseItem.py
#
# Copyright (c) 2006 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMallRegistry.IMallItem import IMallItem
from Products.PloneMallItem.config import *

# additional imports from tagged value 'import'
from Products.PloneMallRegistry.Widget import DecimalL10nWidget

##code-section module-header #fill in your manual code here
from Products.CMFCore.utils import getToolByName
from Products.PloneMallRegistry.Currency import Currency, VOC_CURRENCY
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry

##/code-section module-header

schema = Schema((

    StringField(
        name='currency',
        widget=SelectionWidget
        (
            label='Currency',
            label_msgid='PloneMallItem_label_currency',
            i18n_domain='PloneMallItem',
        ),
        required=1,
        vocabulary=VOC_CURRENCY,
        default_method="getDefaultCurrency"
    ),

    FixedPointField(
        name='price',
        default="0.00",
        widget=DecimalL10nWidget
        (
            label='Price',
            label_msgid='PloneMallItem_label_price',
            i18n_domain='PloneMallItem',
        ),
        required=0,
        accessor="Price",
        validators=('isDecimalL10n',)
    ),

    StringField(
        name='sku_code',
        widget=StringWidget(
            label="SKU Code",
            label_msgid='PloneMallItem_label_sku_code',
            i18n_domain='PloneMallItem',
        ),
        required=1,
        searchable=1
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

BaseItem_schema = schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class BaseItem(BaseContent):
    """Base Item
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),) + (IMallItem,)

    allowed_content_types = []
    _at_rename_after_creation = True

    schema = BaseItem_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    # Methods from Interface IMallItem

    security.declarePublic('getIdentifier')
    def getIdentifier(self):
        """
        returns an identifier string for example to be used in CartItem
        """
        return self.Title()

    security.declareProtected("View", 'getPrice')
    def getPrice(self):
        '''
        Return the product price.
        '''

        return Currency(self.getCurrency(), self.Price(), context=self)

    security.declareProtected("View", 'getSKUCode')
    def getSKUCode(self):
        '''
        Return the product SKU code.

        " The Free On-line Dictionary of Computing (27 SEP 03) "

        SKU

        stock-keeping unit
        '''

        return self.getSku_code()

    security.declareProtected("View", 'isPurchaseableProduct')
    def isPurchaseableProduct(self):
        '''
        Return true if the item is a purchaseable product.
        '''

        reg = getToolByName(self, 'portal_mall_registry')
        return reg.objectSupportsInterface(self, IMallItem)

    # Manually created methods

    def getDefaultCurrency(self):
        if hasattr(self,'aq_parent'):
            reg = getToolByName(self, 'portal_mall_registry')
            return reg.getDefaultCurrency()
        else:
            return None

    def setCategories(self,categories):
        self.deleteReferences('category')
        for cat in categories:
            if not cat is None:
                self.addReference(cat,'category')


# end of class BaseItem

##code-section module-footer #fill in your manual code here
##/code-section module-footer



