# -*- coding: utf-8 -*-
#
# File: PaymentInformation.py
#
# Copyright (c) 2007 by BlueDynamics Alliance (Austria), Weimar Desenvolvimento
# e Consultoria em Informatica - Ltda (Brasil), Zettai.net LCC (United States)
# Generator: ArchGenXML Version 1.5.2
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>, Phil Auersperg <phil@bluedynamics.com>,
Fabiano Weimar dos Santos <xiru@xiru.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PloneMallRegistry.IPaymentInformation import IPaymentInformation
from Products.PloneMallPayment.config import *

##code-section module-header #fill in your manual code here
from Products.PloneMallRegistry.Currency import Currency, VOC_CURRENCY
from config import *
from Products.PloneMallRegistry.MallRegistry import FactoryRegistry
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.debug import deprecated
##/code-section module-header

schema = Schema((

    StringField(
        name='method_name',
        widget=StringWidget(
            label="Method Name",
            label_msgid='PloneMallPayment_label_method_name',
            i18n_domain='PloneMallPayment',
        ),
        required=1
    ),

    StringField(
        name='currency',
        widget=SelectionWidget
        (
            label='Currency',
            label_msgid='PloneMallPayment_label_currency',
            i18n_domain='PloneMallPayment',
        ),
        required=1,
        vocabulary=VOC_CURRENCY,
        default_method="getDefaultCurrency"
    ),

    FixedPointField(
        name='value',
        widget=DecimalWidget(
            label='Value',
            label_msgid='PloneMallPayment_label_value',
            i18n_domain='PloneMallPayment',
        ),
        required=0
    ),

    BooleanField(
        name='confirmed',
        widget=BooleanField._properties['widget'](
            description="The payment was confirmed?",
            label='Confirmed',
            label_msgid='PloneMallPayment_label_confirmed',
            description_msgid='PloneMallPayment_help_confirmed',
            i18n_domain='PloneMallPayment',
        )
    ),

    ReferenceField(
        name='payment_settings',
        widget=ReferenceWidget(
            label='Payment_settings',
            label_msgid='PloneMallPayment_label_payment_settings',
            i18n_domain='PloneMallPayment',
        ),
        allowed_types=('IPaymentSettings',),
        multiValued=0,
        relationship='paymentinformation_payment_settings'
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

PaymentInformation_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class PaymentInformation(BaseContent):
    """Payment Information
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),) + (IPaymentInformation,)

    # This name appears in the 'add' box
    archetype_name = 'PaymentInformation'

    meta_type = 'PaymentInformation'
    portal_type = 'PaymentInformation'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 0
    #content_icon = 'PaymentInformation.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "PaymentInformation"
    typeDescMsgId = 'description_edit_paymentinformation'

    _at_rename_after_creation = False

    schema = PaymentInformation_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header

    # Methods

    # Methods from Interface IPaymentInformation

    security.declarePublic('processPayment')
    def processPayment(self):
        """

        """
        self.portal_workflow.doActionFor(self.getOrder(),'finish')
        return self.REQUEST.RESPONSE.redirect(self.aq_parent.absolute_url()+ '/afterCheckout')

    security.declareProtected("View", 'isPaymentConfirmed')
    def isPaymentConfirmed(self):
        """
        Return true when the payment is confirmed
        """
        return self.getConfirmed()

    security.declarePublic('getSettings')
    def getSettings(self):
        """
        Return the payment settings instance corresponding to the payment method.
        """
        tool=getToolByName(self,'portal_paymentsettings')
        for o in tool.objectValues():
            if o.getPayment_method() == self.getMethodName():
                return o

    security.declareProtected("View", 'getPaymentTotal')
    def getPaymentTotal(self):
        '''

        '''
        return Currency(self.getCurrency(), self.getValue(), context=self)

    security.declareProtected("View", 'isPaymentProcessed')
    def isPaymentProcessed(self):
        '''

        '''
        deprecated('Superseeded by workflow')
        return None

    security.declareProtected("View", 'getMethodName')
    def getMethodName(self):
        '''

        '''
        return self.getMethod_name()

    # Manually created methods

    security.declarePublic('getPaymentMethod')
    def getPaymentMethod(self):
        """

        """
        deprecated('Not used.')
        return None

    security.declarePublic('getPaymentSettings')
    def getPaymentSettings(self):
        """

        """
        deprecated('Use getSettings instead (from IPaymentInformation)')
        return self.getSettings()

    security.declarePublic('getPaymentMethodName')
    def getPaymentMethodName(self):
        """

        """
        deprecated('Use getMethodName instead (from IPaymentInformation)')
        return self.getMethodName()

    def initiatePaymentProcess(self):
        """

        """

    def getDefaultCurrency(self):
        return self.portal_mall_registry.getDefaultCurrency()

    security.declareProtected("View", 'isPaymentConfirmated')
    def isPaymentConfirmated(self):
        '''

        '''
        deprecated('Spelling error, renamed to isPaymentConfirmed ')
        return self.getConfirmed()



registerType(PaymentInformation, PROJECTNAME)
# end of class PaymentInformation

##code-section module-footer #fill in your manual code here
if register_payment:
    FactoryRegistry.registerMallFactory('PaymentInformation', PaymentInformation, (IPaymentInformation,))
##/code-section module-footer



