from fixedpoint import FixedPoint

try:
   from Products.Archetypes.public import DisplayList
   from Products.CMFPlone.i18nl10n import utranslate
# TODO: We need a exchange service to convert one currency to another.
# TODO: We need a tool to represent currency codes (if possible, by country). See ISO-4217 specification.

   VOC_CURRENCY = DisplayList((
      ('USD', 'US$'), # U.S. Dollars
      ('EUR', 'Eur'), # Euros
#  ('GBP', 'GBP'), # Pounds Sterling
#  ('CAD', 'CAD'), # Canadian Dollars
#  ('JPY', 'JPY'), # Yen
    ))
except ImportError:
    pass

class Currency:
    '''
    FixedPoint wrapper to represent Currency
    '''

    def __init__(self, currency = None, value = '0.00', context=None):
        self._currency = currency
        self._value = FixedPoint(value)
        self._context = context

    def compareCurrency(self,c1,c2):
        if c1: c1=c1.lower()
        if c2: c2=c2.lower()
        return c1==c2

    def __add__(self, other):
        
        # if one of the summands has no currency set, lets set it
        # automatically by the other's currency
        currency=self._currency or other._currency

        if type(self) != type(other):
            raise TypeError("You should sum a Currency object with another Currency object.")

        if self._currency and other._currency and not self.compareCurrency(self._currency , other._currency):
            raise TypeError("You should not sum different kinds of currency.")

        return Currency(currency, str(self._value + other._value), self._context)

    def __mul__(self, other):
        return Currency(self._currency, str(self._value * FixedPoint(str(other))), self._context)

    def getCurrencySymbol(self):
        '''
        Return the currency symbol.
        '''
        return self._currency

    def getValue(self):
        '''
        Return the currency value.
        '''
        return str(self._value)

    def getFormatedValue(self):
        '''
        Return a formated string with the currency symbol and the
        currency value.
        '''
        if self._context:
            # localise the value
            decimal_symbol = self._context.translate('mall_decimal_symbol',
                                       domain='PloneMallRegistry', default='.')
            # TODO: translation of currency codes should happen in the vocab
            translated_currency_code = self._context.translate(self._currency,
                                       domain='PloneMallRegistry',
                                       default=VOC_CURRENCY.getValue(self._currency)
                                                               )
            # localise the placement of currency code and value
            translated_currency_repr = self._context.translate('mall_currency_repr',
                                       mapping = {'value' : str(self._value).replace('.', decimal_symbol),
                                                  'currency' : translated_currency_code},
                                       domain='PloneMallRegistry',
                                       default = "%s %s" % (translated_currency_code,
                                                 str(self._value).replace('.', decimal_symbol))
                                                               )
            return translated_currency_repr
        else:
            return "%s %s" % (VOC_CURRENCY.getValue(self._currency),self._value)

    def __repr__(self):
        return self.getFormatedValue()

    def __float__(self):
        return float(self.getValue())

if __name__ == '__main__':
    c1 = Currency('USD', '10.99')
    print c1
    c2 = Currency('USD', '12.00')
    print c2
    print c1 + c2
    print c1 * 3
    print c1 * 3.0
    print c1 * '3'
    print c1 * '3.0'

    c2 = Currency('USD','100')
    print 'float(c2):',float(c2)
